#!/usr/bin/env python3
"""
Security Benchmark Tool
Advanced security configuration assessment and benchmarking tool.
Parameters:
    - target: Target system or configuration
    - profile: Benchmark profile to use
    - output: Output file for results
Requires: psutil, winreg (Windows), pwd, grp (Linux) packages
"""
import argparse
import logging
from datetime import datetime
import json
import os
import sys
import platform
import subprocess
import socket
from pathlib import Path
import psutil
try:
    import winreg
except ImportError:
    winreg = None

class SecurityBenchmark:
    def __init__(self, target, profile='standard'):
        self.target = target
        self.profile = profile.lower()
        self.os_type = platform.system().lower()
        self.results = {}
        self.setup_logging()
        self.load_benchmarks()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def load_benchmarks(self):
        """Load security benchmarks"""
        self.benchmarks = {
            'windows': {
                'account_policies': {
                    'password_policy': [
                        'MinimumPasswordLength',
                        'PasswordComplexity',
                        'MaximumPasswordAge',
                        'MinimumPasswordAge',
                        'PasswordHistorySize'
                    ],
                    'account_lockout': [
                        'LockoutDuration',
                        'LockoutThreshold',
                        'ResetLockoutCount'
                    ]
                },
                'security_options': {
                    'network_security': [
                        'LmCompatibilityLevel',
                        'RestrictAnonymous',
                        'NoLMHash'
                    ],
                    'audit_policy': [
                        'AuditLogonEvents',
                        'AuditObjectAccess',
                        'AuditPrivilegeUse'
                    ]
                },
                'system_services': [
                    'RemoteRegistry',
                    'TelnetServer',
                    'SNMP'
                ],
                'registry_settings': {
                    'HKLM\\System\\CurrentControlSet\\Control\\Lsa': [
                        'RestrictAnonymousSAM',
                        'RestrictAnonymous',
                        'EveryoneIncludesAnonymous'
                    ],
                    'HKLM\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\System': [
                        'DisableCAD',
                        'EnableLUA',
                        'ConsentPromptBehaviorAdmin'
                    ]
                }
            },
            'linux': {
                'authentication': {
                    'password_quality': [
                        'PASS_MAX_DAYS',
                        'PASS_MIN_DAYS',
                        'PASS_MIN_LEN',
                        'PASS_WARN_AGE'
                    ],
                    'login_settings': [
                        'PermitRootLogin',
                        'AllowUsers',
                        'DenyUsers'
                    ]
                },
                'file_permissions': {
                    'critical_files': [
                        '/etc/passwd',
                        '/etc/shadow',
                        '/etc/group',
                        '/etc/gshadow'
                    ],
                    'home_directories': [
                        'UserDirMode',
                        'CreateHomeDir'
                    ]
                },
                'system_settings': {
                    'kernel_parameters': [
                        'net.ipv4.ip_forward',
                        'net.ipv4.conf.all.accept_redirects',
                        'net.ipv4.conf.all.accept_source_route'
                    ],
                    'process_accounting': [
                        'audit_enabled',
                        'audit_backlog_limit'
                    ]
                },
                'network_security': {
                    'firewall_rules': [
                        'INPUT',
                        'OUTPUT',
                        'FORWARD'
                    ],
                    'network_parameters': [
                        'tcp_syncookies',
                        'tcp_max_syn_backlog',
                        'tcp_synack_retries'
                    ]
                }
            }
        }

    def check_windows_security(self):
        """Check Windows security settings"""
        if not winreg:
            return None

        results = {
            'account_policies': {},
            'security_options': {},
            'system_services': {},
            'registry_settings': {}
        }

        try:
            # Check account policies
            results['account_policies'] = self._check_windows_account_policies()

            # Check security options
            results['security_options'] = self._check_windows_security_options()

            # Check system services
            results['system_services'] = self._check_windows_services()

            # Check registry settings
            results['registry_settings'] = self._check_windows_registry()

        except Exception as e:
            self.logger.error(f"Error checking Windows security: {str(e)}")

        return results

    def _check_windows_account_policies(self):
        """Check Windows account policies"""
        policies = {}
        try:
            key = winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE,
                r"SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\System",
                0,
                winreg.KEY_READ
            )
            
            for policy in self.benchmarks['windows']['account_policies']['password_policy']:
                try:
                    value, _ = winreg.QueryValueEx(key, policy)
                    policies[policy] = value
                except:
                    policies[policy] = None

            winreg.CloseKey(key)
        except Exception as e:
            self.logger.error(f"Error checking account policies: {str(e)}")
        
        return policies

    def _check_windows_security_options(self):
        """Check Windows security options"""
        options = {}
        try:
            for category, settings in self.benchmarks['windows']['security_options'].items():
                options[category] = {}
                for setting in settings:
                    try:
                        result = subprocess.run(
                            ['secedit', '/export', '/cfg', 'temp.cfg'],
                            capture_output=True,
                            text=True
                        )
                        with open('temp.cfg', 'r') as f:
                            content = f.read()
                            if setting in content:
                                options[category][setting] = True
                            else:
                                options[category][setting] = False
                        os.remove('temp.cfg')
                    except:
                        options[category][setting] = None
        except Exception as e:
            self.logger.error(f"Error checking security options: {str(e)}")
        
        return options

    def _check_windows_services(self):
        """Check Windows services status"""
        services = {}
        try:
            for service in self.benchmarks['windows']['system_services']:
                try:
                    result = subprocess.run(
                        ['sc', 'query', service],
                        capture_output=True,
                        text=True
                    )
                    services[service] = 'RUNNING' in result.stdout
                except:
                    services[service] = None
        except Exception as e:
            self.logger.error(f"Error checking services: {str(e)}")
        
        return services

    def _check_windows_registry(self):
        """Check Windows registry settings"""
        settings = {}
        try:
            for path, values in self.benchmarks['windows']['registry_settings'].items():
                settings[path] = {}
                try:
                    key = winreg.OpenKey(
                        winreg.HKEY_LOCAL_MACHINE,
                        path,
                        0,
                        winreg.KEY_READ
                    )
                    for value in values:
                        try:
                            data, _ = winreg.QueryValueEx(key, value)
                            settings[path][value] = data
                        except:
                            settings[path][value] = None
                    winreg.CloseKey(key)
                except:
                    for value in values:
                        settings[path][value] = None
        except Exception as e:
            self.logger.error(f"Error checking registry: {str(e)}")
        
        return settings

    def check_linux_security(self):
        """Check Linux security settings"""
        results = {
            'authentication': {},
            'file_permissions': {},
            'system_settings': {},
            'network_security': {}
        }

        try:
            # Check authentication settings
            results['authentication'] = self._check_linux_authentication()

            # Check file permissions
            results['file_permissions'] = self._check_linux_file_permissions()

            # Check system settings
            results['system_settings'] = self._check_linux_system_settings()

            # Check network security
            results['network_security'] = self._check_linux_network_security()

        except Exception as e:
            self.logger.error(f"Error checking Linux security: {str(e)}")

        return results

    def _check_linux_authentication(self):
        """Check Linux authentication settings"""
        auth_settings = {}
        try:
            # Check password quality settings
            with open('/etc/login.defs', 'r') as f:
                content = f.read()
                auth_settings['password_quality'] = {}
                for setting in self.benchmarks['linux']['authentication']['password_quality']:
                    match = re.search(f'{setting}\\s+(\\d+)', content)
                    if match:
                        auth_settings['password_quality'][setting] = int(match.group(1))
                    else:
                        auth_settings['password_quality'][setting] = None

            # Check SSH login settings
            if os.path.exists('/etc/ssh/sshd_config'):
                with open('/etc/ssh/sshd_config', 'r') as f:
                    content = f.read()
                    auth_settings['login_settings'] = {}
                    for setting in self.benchmarks['linux']['authentication']['login_settings']:
                        match = re.search(f'{setting}\\s+(.*)', content)
                        if match:
                            auth_settings['login_settings'][setting] = match.group(1)
                        else:
                            auth_settings['login_settings'][setting] = None

        except Exception as e:
            self.logger.error(f"Error checking authentication: {str(e)}")
        
        return auth_settings

    def _check_linux_file_permissions(self):
        """Check Linux file permissions"""
        permissions = {}
        try:
            # Check critical files
            permissions['critical_files'] = {}
            for file in self.benchmarks['linux']['file_permissions']['critical_files']:
                try:
                    stat = os.stat(file)
                    permissions['critical_files'][file] = {
                        'mode': stat.st_mode & 0o777,
                        'uid': stat.st_uid,
                        'gid': stat.st_gid
                    }
                except:
                    permissions['critical_files'][file] = None

            # Check home directories
            permissions['home_directories'] = {}
            try:
                result = subprocess.run(
                    ['useradd', '-D'],
                    capture_output=True,
                    text=True
                )
                for setting in self.benchmarks['linux']['file_permissions']['home_directories']:
                    match = re.search(f'{setting}=(.*)', result.stdout)
                    if match:
                        permissions['home_directories'][setting] = match.group(1)
                    else:
                        permissions['home_directories'][setting] = None
            except:
                for setting in self.benchmarks['linux']['file_permissions']['home_directories']:
                    permissions['home_directories'][setting] = None

        except Exception as e:
            self.logger.error(f"Error checking file permissions: {str(e)}")
        
        return permissions

    def _check_linux_system_settings(self):
        """Check Linux system settings"""
        settings = {}
        try:
            # Check kernel parameters
            settings['kernel_parameters'] = {}
            for param in self.benchmarks['linux']['system_settings']['kernel_parameters']:
                try:
                    result = subprocess.run(
                        ['sysctl', param],
                        capture_output=True,
                        text=True
                    )
                    if result.returncode == 0:
                        value = result.stdout.split('=')[1].strip()
                        settings['kernel_parameters'][param] = value
                    else:
                        settings['kernel_parameters'][param] = None
                except:
                    settings['kernel_parameters'][param] = None

            # Check process accounting
            settings['process_accounting'] = {}
            try:
                result = subprocess.run(
                    ['auditctl', '-s'],
                    capture_output=True,
                    text=True
                )
                for setting in self.benchmarks['linux']['system_settings']['process_accounting']:
                    match = re.search(f'{setting}\\s+(\\d+)', result.stdout)
                    if match:
                        settings['process_accounting'][setting] = int(match.group(1))
                    else:
                        settings['process_accounting'][setting] = None
            except:
                for setting in self.benchmarks['linux']['system_settings']['process_accounting']:
                    settings['process_accounting'][setting] = None

        except Exception as e:
            self.logger.error(f"Error checking system settings: {str(e)}")
        
        return settings

    def _check_linux_network_security(self):
        """Check Linux network security settings"""
        security = {}
        try:
            # Check firewall rules
            security['firewall_rules'] = {}
            try:
                result = subprocess.run(
                    ['iptables', '-L'],
                    capture_output=True,
                    text=True
                )
                for chain in self.benchmarks['linux']['network_security']['firewall_rules']:
                    if chain in result.stdout:
                        security['firewall_rules'][chain] = True
                    else:
                        security['firewall_rules'][chain] = False
            except:
                for chain in self.benchmarks['linux']['network_security']['firewall_rules']:
                    security['firewall_rules'][chain] = None

            # Check network parameters
            security['network_parameters'] = {}
            for param in self.benchmarks['linux']['network_security']['network_parameters']:
                try:
                    result = subprocess.run(
                        ['sysctl', f'net.ipv4.{param}'],
                        capture_output=True,
                        text=True
                    )
                    if result.returncode == 0:
                        value = result.stdout.split('=')[1].strip()
                        security['network_parameters'][param] = value
                    else:
                        security['network_parameters'][param] = None
                except:
                    security['network_parameters'][param] = None

        except Exception as e:
            self.logger.error(f"Error checking network security: {str(e)}")
        
        return security

    def generate_report(self):
        """Generate benchmark report"""
        try:
            report = {
                'timestamp': datetime.now().isoformat(),
                'target': self.target,
                'profile': self.profile,
                'system': {
                    'os': platform.system(),
                    'version': platform.version(),
                    'architecture': platform.machine()
                }
            }

            # Get security check results
            if self.os_type == 'windows':
                self.results = self.check_windows_security()
            else:
                self.results = self.check_linux_security()

            report['results'] = self.results

            # Calculate compliance scores
            scores = self._calculate_compliance()
            report['compliance'] = scores

            return report
        except Exception as e:
            self.logger.error(f"Error generating report: {str(e)}")
            return None

    def _calculate_compliance(self):
        """Calculate compliance scores"""
        scores = {
            'total_checks': 0,
            'passed_checks': 0,
            'failed_checks': 0,
            'not_applicable': 0,
            'categories': {}
        }

        try:
            def process_results(results, category=''):
                for key, value in results.items():
                    if isinstance(value, dict):
                        cat = f"{category}/{key}" if category else key
                        scores['categories'][cat] = {
                            'total': 0,
                            'passed': 0,
                            'failed': 0,
                            'na': 0
                        }
                        process_results(value, cat)
                    else:
                        scores['total_checks'] += 1
                        if category:
                            scores['categories'][category]['total'] += 1
                        
                        if value is None:
                            scores['not_applicable'] += 1
                            if category:
                                scores['categories'][category]['na'] += 1
                        elif value is True:
                            scores['passed_checks'] += 1
                            if category:
                                scores['categories'][category]['passed'] += 1
                        else:
                            scores['failed_checks'] += 1
                            if category:
                                scores['categories'][category]['failed'] += 1

            process_results(self.results)

            # Calculate percentages
            total_applicable = (
                scores['total_checks'] - scores['not_applicable']
            )
            if total_applicable > 0:
                scores['compliance_percentage'] = (
                    scores['passed_checks'] / total_applicable * 100
                )
                
                for category in scores['categories'].values():
                    total_applicable = (
                        category['total'] - category['na']
                    )
                    if total_applicable > 0:
                        category['compliance_percentage'] = (
                            category['passed'] / total_applicable * 100
                        )
                    else:
                        category['compliance_percentage'] = 0
            else:
                scores['compliance_percentage'] = 0

        except Exception as e:
            self.logger.error(f"Error calculating compliance: {str(e)}")

        return scores

    def save_report(self, report, output_file):
        """Save benchmark report to file"""
        try:
            with open(output_file, 'w') as f:
                json.dump(report, f, indent=4)
            self.logger.info(f"Report saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving report: {str(e)}")

    def print_summary(self, report):
        """Print benchmark summary"""
        self.logger.info("\nSecurity Benchmark Summary:")
        self.logger.info(f"Target: {report['target']}")
        self.logger.info(f"Profile: {report['profile']}")
        self.logger.info(
            f"Operating System: {report['system']['os']} "
            f"{report['system']['version']}"
        )
        
        if 'compliance' in report:
            compliance = report['compliance']
            self.logger.info("\nCompliance Summary:")
            self.logger.info(
                f"Overall Compliance: "
                f"{compliance['compliance_percentage']:.1f}%"
            )
            self.logger.info(f"Total Checks: {compliance['total_checks']}")
            self.logger.info(
                f"Passed: {compliance['passed_checks']}"
            )
            self.logger.info(
                f"Failed: {compliance['failed_checks']}"
            )
            self.logger.info(
                f"Not Applicable: {compliance['not_applicable']}"
            )
            
            self.logger.info("\nCategory Compliance:")
            for category, scores in compliance['categories'].items():
                if scores['total'] > 0:
                    self.logger.info(
                        f"\n{category}:"
                    )
                    self.logger.info(
                        f"Compliance: {scores['compliance_percentage']:.1f}%"
                    )
                    self.logger.info(
                        f"Passed: {scores['passed']}/{scores['total']}"
                    )

    def start_benchmark(self, output_file=None):
        """Start security benchmark assessment"""
        self.logger.info("Starting security benchmark assessment")
        self.logger.info(f"Target: {self.target}")
        self.logger.info(f"Profile: {self.profile}")
        
        try:
            # Generate and save report
            report = self.generate_report()
            if report and output_file:
                self.save_report(report, output_file)
            if report:
                self.print_summary(report)
                
        except KeyboardInterrupt:
            self.logger.info("Benchmark interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during benchmark: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Security Benchmark Tool"
    )
    parser.add_argument(
        "target",
        help="Target system or configuration"
    )
    parser.add_argument(
        "--profile",
        choices=['standard', 'high'],
        default='standard',
        help="Benchmark profile to use"
    )
    parser.add_argument(
        "--output",
        help="Output file for benchmark report"
    )
    args = parser.parse_args()

    benchmark = SecurityBenchmark(args.target, args.profile)
    benchmark.start_benchmark(args.output)
